@@ -24,7 +24,7 @@
 #define FILE_UTILS_H
 
 #include <time.h>
-#include <libgnomevfs/gnome-vfs-types.h>
+#include <libgnomevfs/gnome-vfs-file-size.h>
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
 
 
@@ -48,11 +48,12 @@ gboolean            file_move                    (const gchar *from,
 
 gint                file_in_path                 (const gchar *name);
 
-gboolean            ensure_dir_exists            (const gchar *a_path);
+gboolean            ensure_dir_exists            (const gchar *a_path,
+						  mode_t       mode);
 
 gboolean            file_is_hidden               (const gchar *name);
 
-const gchar *       file_name_from_path          (const gchar *path);
+G_CONST_RETURN gchar *             file_name_from_path          (const gchar *path);
 
 gchar *             remove_level_from_path       (const gchar *path);
 
@@ -60,8 +61,8 @@ gchar *             remove_extension_from_path   (const gchar *path);
 
 gchar *             remove_ending_separator      (const gchar *path);
 
-gboolean            file_extension_is            (char *filename, 
-						  char *ext);
+gboolean            file_extension_is            (const char *filename, 
+						  const char *ext);
 
 void                path_list_free               (GList *path_list);
 
@@ -74,4 +75,9 @@ gchar *             shell_escape                 (const gchar *filename);
 
 gchar *             application_get_command      (const GnomeVFSMimeApplication *app);
 
+gboolean            match_patterns               (char       **patterns, 
+						  const char  *string);
+
+char **             search_util_get_patterns     (const char  *pattern_string);
+
 #endif /* FILE_UTILS_H */
