@@ -35,12 +35,12 @@
 #include "fr-command-rar.h"
 #include "fr-command-tar.h"
 #include "fr-command-zip.h"
+#include "fr-marshal.h"
 #include "fr-process.h"
 
-
 #define MAX_CHUNK_LEN 16000 /* FIXME : what is the max length of a command 
 			     * line ? */
-
+#define UNKNOWN_TYPE "application/octet-stream"
 
 enum {
 	START,
@@ -48,13 +48,95 @@ enum {
 	LAST_SIGNAL
 };
 
-
-static GtkObjectClass *parent_class;
+static GObjectClass *parent_class;
 static guint fr_archive_signals[LAST_SIGNAL] = { 0 };
 
+static void fr_archive_class_init (FRArchiveClass *class);
+static void fr_archive_init       (FRArchive *archive);
+static void fr_archive_finalize   (GObject *object);
+
+
+GType
+fr_archive_get_type (void)
+{
+        static GType type = 0;
+
+        if (! type) {
+                static const GTypeInfo type_info = {
+			sizeof (FRArchiveClass),
+			NULL,
+			NULL,
+			(GClassInitFunc) fr_archive_class_init,
+			NULL,
+			NULL,
+			sizeof (FRArchive),
+			0,
+			(GInstanceInitFunc) fr_archive_init
+		};
+
+		type = g_type_register_static (G_TYPE_OBJECT,
+					       "FRArchive",
+					       &type_info,
+					       0);
+        }
+
+        return type;
+}
+
 
 static void
-fr_archive_destroy (GtkObject *object)
+fr_archive_class_init (FRArchiveClass *class)
+{
+        GObjectClass *gobject_class = G_OBJECT_CLASS (class);
+
+        parent_class = g_type_class_peek_parent (class);
+
+	fr_archive_signals[START] =
+                g_signal_new ("start",
+			      G_TYPE_FROM_CLASS (class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (FRArchiveClass, start),
+			      NULL, NULL,
+			      fr_marshal_VOID__INT,
+			      G_TYPE_NONE, 
+			      1, G_TYPE_INT);
+	fr_archive_signals[DONE] =
+                g_signal_new ("done",
+			      G_TYPE_FROM_CLASS (class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (FRArchiveClass, done),
+			      NULL, NULL,
+			      fr_marshal_VOID__INT_POINTER,
+			      G_TYPE_NONE, 2,
+			      G_TYPE_INT,
+			      G_TYPE_POINTER);
+	
+	gobject_class->finalize = fr_archive_finalize;
+	class->start = NULL;
+	class->done = NULL;
+}
+
+
+static void
+fr_archive_init (FRArchive *archive)
+{
+	archive->filename = NULL;
+	archive->command = NULL;
+	archive->process = fr_process_new ();
+	archive->is_compressed_file = FALSE;
+	archive->fake_load = FALSE;
+}
+
+
+FRArchive *
+fr_archive_new ()
+{
+	return FR_ARCHIVE (g_object_new (FR_TYPE_ARCHIVE, NULL));
+}
+
+
+static void
+fr_archive_finalize (GObject *object)
 {
 	FRArchive *archive;
 
@@ -67,104 +149,24 @@ fr_archive_destroy (GtkObject *object)
 		g_free (archive->filename);
 
 	if (archive->command != NULL) 
-		gtk_object_unref (GTK_OBJECT (archive->command));
+		g_object_unref (archive->command);
 
-	gtk_object_unref (GTK_OBJECT (archive->process));
+	g_object_unref (archive->process);
 
 	/* Chain up */
 
-        if (GTK_OBJECT_CLASS (parent_class)->destroy)
-                (* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
+        if (G_OBJECT_CLASS (parent_class)->finalize)
+                G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 
-static void
-fr_archive_class_init (FRArchiveClass *class)
-{
-        GtkObjectClass *object_class;
-
-        object_class = (GtkObjectClass *) class;
-        parent_class = gtk_type_class (gtk_object_get_type ());
-
-	fr_archive_signals[START] =
-                gtk_signal_new ("start",
-                                GTK_RUN_LAST,
-                                object_class->type,
-                                GTK_SIGNAL_OFFSET (FRArchiveClass, start),
-                                gtk_marshal_NONE__INT,
-                                GTK_TYPE_NONE, 1,
-				GTK_TYPE_INT);
-
-	fr_archive_signals[DONE] =
-                gtk_signal_new ("done",
-                                GTK_RUN_LAST,
-                                object_class->type,
-                                GTK_SIGNAL_OFFSET (FRArchiveClass, done),
-                                gtk_marshal_NONE__INT_POINTER,
-                                GTK_TYPE_NONE, 2,
-                                GTK_TYPE_INT,
-				GTK_TYPE_POINTER);
-        gtk_object_class_add_signals (object_class, fr_archive_signals, 
-                                      LAST_SIGNAL);
-
-	object_class->destroy = fr_archive_destroy;
-	class->start = NULL;
-	class->done  = NULL;
-}
-
-
-static void
-fr_archive_init (FRArchive *archive)
-{
-	archive->filename = NULL;
-	archive->command = NULL;
-	archive->process = fr_process_new ();
-	archive->is_compressed_file = FALSE;
-}
-
-
-GtkType
-fr_archive_get_type (void)
-{
-        static GtkType fr_archive_type = 0;
-
-        if (! fr_archive_type) {
-                GtkTypeInfo fr_archive_info = {
-                        "FRArchive",
-                        sizeof (FRArchive),
-                        sizeof (FRArchiveClass),
-                        (GtkClassInitFunc) fr_archive_class_init,
-                        (GtkObjectInitFunc) fr_archive_init,
-                        NULL, /* reserved_1 */
-                        NULL, /* reserved_2 */
-                        (GtkClassInitFunc) NULL
-                };
-
-                fr_archive_type = gtk_type_unique (gtk_object_get_type (), 
-						   &fr_archive_info);
-        }
-
-        return fr_archive_type;
-}
-
-
-FRArchive *
-fr_archive_new ()
-{
-	FRArchive *archive;
-	archive = FR_ARCHIVE (gtk_type_new (fr_archive_get_type ()));
-	return archive;
-}
-
-
-/* filename must not be escaped. */
 static gboolean
-create_command_from_file_mime_type (FRArchive * archive, char *filename) {
-	const char* mime_type;
-	
+create_command_from_mime_type (FRArchive  *archive, 
+			       const char *filename,
+			       const char *mime_type)
+{
 	archive->is_compressed_file = FALSE;
-	mime_type = gnome_vfs_get_file_mime_type (filename, NULL, FALSE);
-	
+
 	if (is_mime_type (mime_type, "application/x-tar")) {
 		archive->command = fr_command_tar_new (archive->process, 
 						       filename, 
@@ -188,16 +190,86 @@ create_command_from_file_mime_type (FRArchive * archive, char *filename) {
 
 	return TRUE;
 }
+
+
+/* filename must not be escaped. */
+static const char *
+get_mime_type_from_content (const char *filename) 
+{
+	const char *mime_type;
 	
+	mime_type = gnome_vfs_get_file_mime_type (filename, NULL, FALSE);
+	
+	if (strcmp (mime_type, UNKNOWN_TYPE) == 0)
+		return NULL;
+	
+	return mime_type;
+}
+	
+
+static gboolean
+hexcmp (const guchar *first_bytes,
+	const guchar *buffer,
+	int           len)
+{
+	int i;
+
+	for (i = 0; i < len; i++)
+		if (first_bytes[i] != buffer[i])
+			return FALSE;
+
+	return TRUE;
+}
+
+
+/* filename must not be escaped. */
+static const char *
+get_mime_type_from_sniffer (const char *filename) 
+{
+	static struct {
+		const char *mime_type;
+		const char *first_bytes;
+		int         len;
+	}            sniffer_data [] = { 
+		{ "application/zip",                   "\x50\x4B\x03\x04", 4 },
+		/* FIXME
+		{ "application/x-compressed-tar",      "\x1F\x8B\x08\x08", 4 },
+		{ "application/x-bzip-compressed-tar", "\x42\x5A\x68\x39", 4 },
+		*/
+		{ NULL, NULL, 0 } 
+	};
+	FILE        *file;
+	char         buffer[5];
+	int          n, i;
+
+	file = fopen (filename, "rb");
+
+	if (file == NULL) 
+                return NULL;
+
+	n = fread (buffer, sizeof (char), sizeof (buffer) - 1, file);
+	buffer[n] = 0;
+
+	fclose (file);
+
+	for (i = 0; sniffer_data[i].mime_type != NULL; i++) {
+		const char *first_bytes = sniffer_data[i].first_bytes;
+		int          len        = sniffer_data[i].len;
+
+		if (hexcmp (first_bytes, buffer, len)) 
+			return sniffer_data[i].mime_type;
+	}
+
+	return NULL;
+}
+
 
 /* filename must not be escaped. */
 static gboolean
-create_command_from_filename (FRArchive * archive, 
-			      char *filename,
-			      gboolean loading)
+create_command_from_filename (FRArchive  *archive, 
+			      const char *filename,
+			      gboolean    loading)
 {
-	archive->is_compressed_file = FALSE;
-
 	if (file_extension_is (filename, ".tar.gz")
 	    || file_extension_is (filename, ".tgz")) {
 		archive->command = fr_command_tar_new (archive->process, 
@@ -265,20 +337,21 @@ create_command_from_filename (FRArchive * archive,
 
 static void
 action_started (FRCommand *command, 
-		FRAction action,
+		FRAction   action,
 		FRArchive *archive)
 {
-	gtk_signal_emit (GTK_OBJECT (archive), 
-			 fr_archive_signals[START],
-			 action);
+	g_signal_emit (G_OBJECT (archive), 
+		       fr_archive_signals[START],
+		       0,
+		       action);
 }
 
 
 static void
-action_performed (FRCommand *command, 
-		  FRAction action,
+action_performed (FRCommand   *command, 
+		  FRAction     action,
 		  FRProcError *error,
-		  FRArchive *archive)
+		  FRArchive   *archive)
 {
 	char *s_action = NULL;
 
@@ -296,25 +369,39 @@ action_performed (FRCommand *command,
 	case FR_ACTION_EXTRACT:
 		s_action = "Extract";
 		break;
+	case FR_ACTION_TEST:
+		s_action = "Test";
+		break;
 	}
 	g_print ("%s [DONE]\n", s_action);
 #endif
 
-	gtk_signal_emit (GTK_OBJECT (archive), 
-			 fr_archive_signals[DONE],
-			 action,
-			 error);
+	g_signal_emit (G_OBJECT (archive), 
+		       fr_archive_signals[DONE],
+		       0,
+		       action,
+		       error);
 }
 
 
 /* filename must not be escaped. */
-void
-fr_archive_new_file (FRArchive *archive, char *filename)
+gboolean
+fr_archive_new_file (FRArchive  *archive, 
+		     const char *filename)
 {
 	FRCommand *tmp_command;
 
 	if (filename == NULL)
-		return;
+		return FALSE;
+
+	tmp_command = archive->command;
+	if (! create_command_from_filename (archive, filename, FALSE)) {
+		archive->command = tmp_command;
+		return FALSE;
+	}
+
+	if (tmp_command != NULL) 
+		g_object_unref (G_OBJECT (tmp_command));
 
 	archive->read_only = FALSE;
 
@@ -322,27 +409,26 @@ fr_archive_new_file (FRArchive *archive, char *filename)
 		g_free (archive->filename);	
 	archive->filename = g_strdup (filename);
 
-	tmp_command = archive->command;
-	if (! create_command_from_filename (archive, filename, FALSE))
-		return;
-	if (tmp_command != NULL) 
-		gtk_object_unref (GTK_OBJECT (tmp_command));
+	g_signal_connect (G_OBJECT (archive->command), 
+			  "start",
+			  G_CALLBACK (action_started),
+			  archive);
+	g_signal_connect (G_OBJECT (archive->command), 
+			  "done",
+			  G_CALLBACK (action_performed),
+			  archive);
 
-	gtk_signal_connect (GTK_OBJECT (archive->command), "start",
-			    GTK_SIGNAL_FUNC (action_started),
-			    archive);
-	gtk_signal_connect (GTK_OBJECT (archive->command), "done",
-			    GTK_SIGNAL_FUNC (action_performed),
-			    archive);
+	return TRUE;
 }
 
 
 /* filename must not be escaped. */
 gboolean
-fr_archive_load (FRArchive *archive, 
-		 char *filename)
+fr_archive_load (FRArchive  *archive, 
+		 const char *filename)
 {
-	FRCommand *tmp_command;
+	FRCommand  *tmp_command;
+	const char *mime_type;
 
 	g_return_val_if_fail (archive != NULL, FALSE);
 
@@ -359,20 +445,31 @@ fr_archive_load (FRArchive *archive,
 
 	/* prefer mime-magic */
 
-	if (! create_command_from_file_mime_type (archive, filename)) 
-		if (! create_command_from_filename (archive, filename, TRUE))
+	mime_type = get_mime_type_from_sniffer (filename);
+	if (mime_type == NULL)
+		mime_type = get_mime_type_from_content (filename);
+
+	if ((mime_type == NULL)
+	    || ! create_command_from_mime_type (archive, filename, mime_type))
+		if (! create_command_from_filename (archive, filename, TRUE)) {
+			archive->command = tmp_command;
 			return FALSE;
+		}
+
 	if (tmp_command != NULL) 
-		gtk_object_unref (GTK_OBJECT (tmp_command));
+		g_object_unref (G_OBJECT (tmp_command));
 
-	gtk_signal_connect (GTK_OBJECT (archive->command), "start",
-			    GTK_SIGNAL_FUNC (action_started),
-			    archive);
+	g_signal_connect (G_OBJECT (archive->command), 
+			  "start",
+			  G_CALLBACK (action_started),
+			  archive);
 
-	gtk_signal_connect (GTK_OBJECT (archive->command), "done",
-			    GTK_SIGNAL_FUNC (action_performed),
-			    archive);
-	
+	g_signal_connect (G_OBJECT (archive->command), 
+			  "done",
+			  G_CALLBACK (action_performed),
+			  archive);
+
+	archive->command->fake_load = archive->fake_load;
 	fr_command_list (archive->command);
 
 	return TRUE;
@@ -385,22 +482,30 @@ fr_archive_reload (FRArchive *archive)
 	g_return_if_fail (archive != NULL);
 	g_return_if_fail (archive->filename != NULL);
 
+	archive->command->fake_load = archive->fake_load;
 	fr_command_list (archive->command);
 }
 
 
 /* filename must not be escaped. */
 void
-fr_archive_rename (FRArchive *archive,
-		   char *filename)
+fr_archive_rename (FRArchive  *archive,
+		   const char *filename)
 {
 	g_return_if_fail (archive != NULL);
 
-	if (archive->filename != NULL)
-		g_free (archive->filename);
-	archive->filename = g_strdup (filename);
-
-	fr_command_set_filename (archive->command, filename);
+	if (archive->is_compressed_file) 
+		/* If the archive is a compressed file we have to reload it,
+		 * because in this case the 'content' of the archive changes 
+		 * too. */
+		fr_archive_load (archive, filename);
+	else {
+		if (archive->filename != NULL)
+			g_free (archive->filename);
+		archive->filename = g_strdup (filename);
+		
+		fr_command_set_filename (archive->command, filename);
+	}
 }
 
 
@@ -409,7 +514,8 @@ fr_archive_rename (FRArchive *archive,
 
 /* Note: all paths unescaped. */
 static FileData *
-find_file_in_archive (FRArchive *archive, char *path)
+find_file_in_archive (FRArchive *archive, 
+		      char      *path)
 {
 	GList *scan;
 
@@ -445,9 +551,9 @@ escape_file_list (GList *file_list) {
 
 /* Note: all paths unescaped. */
 static GList *
-newer_files_only (FRArchive *archive,
-		  GList *file_list,
-		  gchar *base_dir)
+newer_files_only (FRArchive  *archive,
+		  GList      *file_list,
+		  const char *base_dir)
 {
 	GList *newer_files = NULL;
 	GList *scan;
@@ -482,17 +588,20 @@ newer_files_only (FRArchive *archive,
 
 /* Note: all paths unescaped. */
 void
-fr_archive_add (FRArchive *archive, 
-		GList *file_list, 
-		gchar *base_dir,
-		gboolean update)
+fr_archive_add (FRArchive   *archive, 
+		GList       *file_list, 
+		const char  *base_dir,
+		gboolean     update,
+		const char  *password,
+		FRCompression  compression)
 {
-	GList *new_file_list;
-	gboolean free_new_file_list;
-	GList *e_file_list;
-	GList *scan;
+	GList    *new_file_list;
+	gboolean  free_new_file_list;
+	GList    *e_file_list;
+	GList    *scan;
 
-	g_return_if_fail (file_list != NULL);
+	if (file_list == NULL)
+		return;
 
 	if (archive->read_only)
 		return;
@@ -521,8 +630,8 @@ fr_archive_add (FRArchive *archive,
 	/* when files are already present in a tar archive and are added
 	 * again, they are not replaced, so we have to delete them first. */
 
-	/* if we are adding (== ! update) and add cannot replace or
-	 * if we are updating and add cannot update, 
+	/* if we are adding (== ! update) and 'add' cannot replace or
+	 * if we are updating and 'add' cannot update, 
 	 * delete the files first. */
 
 	if ((! update && ! archive->command->propAddCanReplace)
@@ -550,7 +659,7 @@ fr_archive_add (FRArchive *archive,
 	for (scan = e_file_list; scan != NULL; ) {
 		GList *prev = scan->prev;
 		GList *chunk_list;
-		int l;
+		int    l;
 		
 		chunk_list = scan;
 		l = 0;
@@ -567,7 +676,9 @@ fr_archive_add (FRArchive *archive,
 		fr_command_add (archive->command, 
 				chunk_list, 
 				base_dir, 
-				update);
+				update,
+				password,
+				compression);
 		prev->next = scan;
 	}
 
@@ -575,24 +686,55 @@ fr_archive_add (FRArchive *archive,
 	if (free_new_file_list)
 		g_list_free (new_file_list);
 
-	fr_command_recompress (archive->command);
+	fr_command_recompress (archive->command, compression);
 
-	fr_process_start (archive->process, FALSE);
+	fr_process_start (archive->process);
+}
+
+
+static void
+file_list_remove_from_pattern (GList      **list, 
+			       const char  *pattern)
+{
+	gchar **patterns;
+	GList  *scan;
+	
+	if (pattern == NULL)
+		return;
+	
+	patterns = search_util_get_patterns (pattern);
+	
+	for (scan = *list; scan;) {
+		char *path = scan->data;
+
+		if (match_patterns (patterns, file_name_from_path (path))) {
+			*list = g_list_remove_link (*list, scan);
+			g_free (scan->data);
+			g_list_free (scan);
+			scan = *list;
+		} else
+			scan = scan->next;
+	}
+	
+	g_strfreev (patterns);
 }
 
 
 /* Note: all paths unescaped. */
 void
-fr_archive_add_with_wildcard (FRArchive *archive, 
-			      const char *files,
-			      gchar *base_dir,
-			      gboolean update,
-			      gboolean recursive,
-			      gboolean follow_links,
-			      gboolean same_fs,
-			      gboolean no_backup_files,
-			      gboolean no_dot_files,
-			      gboolean ignore_case)
+fr_archive_add_with_wildcard (FRArchive  *archive, 
+			      const char *include_files,
+			      const char *exclude_files,
+			      const char *base_dir,
+			      gboolean    update,
+			      gboolean    recursive,
+			      gboolean    follow_links,
+			      gboolean    same_fs,
+			      gboolean    no_backup_files,
+			      gboolean    no_dot_files,
+			      gboolean    ignore_case,
+			      const char *password,
+			      FRCompression compression)
 {
 	GList *file_list;
 
@@ -600,27 +742,34 @@ fr_archive_add_with_wildcard (FRArchive *archive,
 		return;
 
 	file_list = get_wildcard_file_list (base_dir, 
-					    files, 
+					    include_files, 
 					    recursive, 
 					    follow_links, 
 					    same_fs,
 					    no_backup_files, 
 					    no_dot_files, 
 					    ignore_case);
+
+	file_list_remove_from_pattern (&file_list, exclude_files);
+
 	fr_archive_add (archive,
 			file_list,
 			base_dir,
-			update);
+			update,
+			password,
+			compression);
 	path_list_free (file_list);
 }
 
 
 /* Note: all paths unescaped. */
 void
-fr_archive_add_directory (FRArchive *archive, 
+fr_archive_add_directory (FRArchive  *archive, 
 			  const char *directory,
-			  gchar *base_dir,
-			  gboolean update)
+			  const char *base_dir,
+			  gboolean    update,
+			  const char *password,
+			  FRCompression compression)
 
 {
 	GList *file_list;
@@ -632,7 +781,9 @@ fr_archive_add_directory (FRArchive *archive,
 	fr_archive_add (archive,
 			file_list,
 			base_dir,
-			update);
+			update,
+			password,
+			compression);
 	path_list_free (file_list);
 }
 
@@ -643,7 +794,7 @@ fr_archive_add_directory (FRArchive *archive,
 /* Note: all paths unescaped. */
 static void
 _archive_remove (FRArchive *archive,
-		 GList *file_list)
+		 GList     *file_list)
 {
 	GList *e_file_list;
 	GList *scan;
@@ -658,7 +809,7 @@ _archive_remove (FRArchive *archive,
 	for (scan = e_file_list; scan != NULL; ) {
 		GList *prev = scan->prev;
 		GList *chunk_list;
-		int l;
+		int    l;
 		
 		chunk_list = scan;
 		l = 0;
@@ -682,8 +833,9 @@ _archive_remove (FRArchive *archive,
 
 /* Note: all paths unescaped. */
 void
-fr_archive_remove (FRArchive *archive,
-		   GList *file_list)
+fr_archive_remove (FRArchive     *archive,
+		   GList         *file_list,
+		   FRCompression  compression)
 {
 	g_return_if_fail (archive != NULL);
 
@@ -693,8 +845,8 @@ fr_archive_remove (FRArchive *archive,
 	fr_process_clear (archive->process);
 	fr_command_uncompress (archive->command);
 	_archive_remove (archive, file_list);
-	fr_command_recompress (archive->command);
-	fr_process_start (archive->process, FALSE);
+	fr_command_recompress (archive->command, compression);
+	fr_process_start (archive->process);
 }
 
 
@@ -703,17 +855,17 @@ fr_archive_remove (FRArchive *archive,
 
 /* Note: all paths escaped, source_dir and dest_dir escaped. */
 static void
-move_files_to_dir (FRArchive *archive,
-		   GList *file_list,
-		   char *source_dir,
-		   char *dest_dir)
+move_files_to_dir (FRArchive  *archive,
+		   GList      *file_list,
+		   const char *source_dir,
+		   const char *dest_dir)
 {
 	GList *scan;
 
 	fr_process_begin_command (archive->process, "mv");
 	fr_process_add_arg (archive->process, "-f");
 	for (scan = file_list; scan; scan = scan->next) {
-		char path[4096]; /* FIXME : 4096 ? */
+		char  path[4096]; /* FIXME : 4096 ? */
 		char *filename = scan->data;
 		
 		if (filename[0] == '/')
@@ -730,15 +882,15 @@ move_files_to_dir (FRArchive *archive,
 
 /* Note: all paths escaped, temp_dir and dest_dir unescaped. */
 static void
-move_files_in_chunks (FRArchive *archive,
-		      GList *file_list,
-		      char *temp_dir,
-		      char *dest_dir)
+move_files_in_chunks (FRArchive  *archive,
+		      GList      *file_list,
+		      const char *temp_dir,
+		      const char *dest_dir)
 {
 	GList *scan;
 	int    e_temp_dir_l;
-	char * e_temp_dir;
-	char * e_dest_dir;
+	char  *e_temp_dir;
+	char  *e_dest_dir;
 
 	e_temp_dir = shell_escape (temp_dir);
 	e_dest_dir = shell_escape (dest_dir);
@@ -747,7 +899,7 @@ move_files_in_chunks (FRArchive *archive,
 	for (scan = file_list; scan != NULL; ) {
 		GList *prev = scan->prev;
 		GList *chunk_list;
-		int l;
+		int    l;
 
 		chunk_list = scan;
 		l = 0;
@@ -775,12 +927,13 @@ move_files_in_chunks (FRArchive *archive,
 
 /* Note: all paths escaped, dest_dir unescaped. */
 static void
-extract_in_chunks (FRCommand *command,
-		   GList *file_list,
-		   char *dest_dir,
-		   gboolean overwrite,
-		   gboolean skip_older,
-		   gboolean junk_paths)
+extract_in_chunks (FRCommand  *command,
+		   GList      *file_list,
+		   const char *dest_dir,
+		   gboolean    overwrite,
+		   gboolean    skip_older,
+		   gboolean    junk_paths,
+		   const char *password)
 {
 	GList *scan;
 
@@ -790,14 +943,15 @@ extract_in_chunks (FRCommand *command,
 				    dest_dir,
 				    overwrite,
 				    skip_older,
-				    junk_paths);
+				    junk_paths,
+				    password);
 		return;
 	}
 
 	for (scan = file_list; scan != NULL; ) {
 		GList *prev = scan->prev;
 		GList *chunk_list;
-		int l;
+		int    l;
 
 		chunk_list = scan;
 		l = 0;
@@ -816,7 +970,8 @@ extract_in_chunks (FRCommand *command,
 				    dest_dir,
 				    overwrite,
 				    skip_older,
-				    junk_paths);
+				    junk_paths,
+				    password);
 		prev->next = scan;
 	}
 }
@@ -826,17 +981,18 @@ extract_in_chunks (FRCommand *command,
  * Note2: Do not escape dest_dir it will escaped in fr_command_extract if 
  *        needed. */
 void
-fr_archive_extract (FRArchive *archive,
-		    GList * file_list,
-		    char * dest_dir,
-		    gboolean skip_older,
-		    gboolean overwrite,
-		    gboolean junk_paths)
+fr_archive_extract (FRArchive  *archive,
+		    GList      *file_list,
+		    const char *dest_dir,
+		    gboolean    skip_older,
+		    gboolean    overwrite,
+		    gboolean    junk_paths,
+		    const char *password)
 {
-	GList *filtered, *e_filtered;
-	GList *scan;
-	gboolean extract_all;
-	gboolean move_to_dest_dir;
+	GList    *filtered, *e_filtered;
+	GList    *scan;
+	gboolean  extract_all;
+	gboolean  move_to_dest_dir;
 
 	g_return_if_fail (archive != NULL);
 
@@ -856,11 +1012,12 @@ fr_archive_extract (FRArchive *archive,
 				   dest_dir,
 				   overwrite,
 				   skip_older,
-				   junk_paths);
+				   junk_paths,
+				   password);
 
 		path_list_free (e_file_list);
 
-		fr_process_start (archive->process, FALSE);
+		fr_process_start (archive->process);
 		return;
 	}
 
@@ -884,9 +1041,9 @@ fr_archive_extract (FRArchive *archive,
 
 	filtered = NULL;
 	for (scan = file_list; scan; scan = scan->next) {
-		FileData *fdata;
-		char *arch_filename = scan->data;
-		char dest_filename[4096];
+		FileData   *fdata;
+		char       *arch_filename = scan->data;
+		char        dest_filename[4096];
 		const char *filename;
 
 		fdata = find_file_in_archive (archive, arch_filename);
@@ -909,7 +1066,7 @@ fr_archive_extract (FRArchive *archive,
 		
 
 #ifdef DEBUG		
-		g_print ("-> %s\n", dest_filename); /* FIXME */
+		g_print ("-> %s\n", dest_filename);
 #endif
 
 		/**/
@@ -950,13 +1107,14 @@ fr_archive_extract (FRArchive *archive,
 					    g_get_tmp_dir (),
 					    "/file-roller.",
 					    getpid ());
-		ensure_dir_exists (temp_dir);
+		ensure_dir_exists (temp_dir, 0700);
 		extract_in_chunks (archive->command,
 				   e_filtered,
 				   temp_dir,
 				   overwrite,
 				   skip_older,
-				   junk_paths);
+				   junk_paths,
+				   password);
 
 		move_files_in_chunks (archive, 
 				      e_filtered, 
@@ -979,7 +1137,8 @@ fr_archive_extract (FRArchive *archive,
 				   dest_dir,
 				   overwrite,
 				   skip_older,
-				   junk_paths);
+				   junk_paths,
+				   password);
 
 	path_list_free (e_filtered);
 	if (filtered != NULL)
@@ -989,18 +1148,36 @@ fr_archive_extract (FRArchive *archive,
 		/* the list has been created in this function. */
 		path_list_free (file_list);
 
-	fr_process_start (archive->process, FALSE);
+	fr_process_start (archive->process);
 }
 
 
-gchar *
-fr_archive_utils_get_file_name_ext (char *filename)
+void
+fr_archive_test (FRArchive  *archive,
+		 const char *password)
+{
+	fr_process_clear (archive->process);
+	fr_command_test (archive->command, password);
+	fr_process_start (archive->process);
+}
+
+
+/*
+ * Remember to keep the ext array in alphanumeric order and to scan the array
+ * in reverse order, this is because the file 'foo.tar.gz' must return the 
+ * '.tar.gz' and not the '.gz' extension.
+ */
+G_CONST_RETURN gchar *
+fr_archive_utils__get_file_name_ext (const char *filename)
 {
 	static char * ext[] = {
-		/* archives */
+		".bz", 
+		".bz2", 
 		".ear",
+		".gz", 
 		".jar",
 		".lzh",
+		".lzo",
 		".rar",
 		".tar", 
 		".tar.bz", 
@@ -1014,20 +1191,35 @@ fr_archive_utils_get_file_name_ext (char *filename)
 		".tgz", 
 		".tzo",
 		".war",
-		".zip",
-		/* single file compressed */
-		".gz", 
 		".z", 
-		".Z", 
-		".bz", 
-		".bz2", 
-		".lzo"};
-	gint i, n;
+		".zip",
+		".Z" 
+	};
+	int n = sizeof (ext) / sizeof (char*);
+	int i;
 
-	n = sizeof (ext) / sizeof (char*);
-	for (i = 0; i < n; i++) 
+	for (i = n - 1; i >= 0; i--) 
 		if (file_extension_is (filename, ext[i]))
 			return ext[i];
 
 	return NULL;
 }
+
+
+gboolean
+fr_archive_utils__file_is_archive (const char *filename)
+{
+	const char *mime_type;
+
+	mime_type = get_mime_type_from_content (filename);
+
+	if (mime_type != NULL)
+		return TRUE;
+
+	mime_type = get_mime_type_from_sniffer (filename);
+
+	if (mime_type != NULL)
+		return TRUE;
+
+	return fr_archive_utils__get_file_name_ext (filename) != NULL;
+}
